"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixIndividualRequirementsTxt = exports.containsRequireDirective = exports.isSupported = exports.pipRequirementsTxt = void 0;
const debugLib = require("debug");
const chalk = require("chalk");
const update_dependencies_1 = require("./update-dependencies");
const missing_remediation_data_1 = require("../../../lib/errors/missing-remediation-data");
const missing_file_name_1 = require("../../../lib/errors/missing-file-name");
const debug = debugLib('snyk-fix:python:requirements.txt');
async function pipRequirementsTxt(entities, options) {
    debug(`Preparing to fix ${entities.length} Python requirements.txt projects`);
    const handlerResult = {
        succeeded: [],
        failed: [],
        skipped: [],
    };
    for (const entity of entities) {
        try {
            const isSupportedResponse = await isSupported(entity);
            if (projectTypeSupported(isSupportedResponse)) {
                const fixedEntity = await fixIndividualRequirementsTxt(entity, options);
                handlerResult.succeeded.push(fixedEntity);
            }
            else {
                handlerResult.skipped.push({
                    original: entity,
                    userMessage: isSupportedResponse.reason,
                });
            }
        }
        catch (e) {
            handlerResult.failed.push({ original: entity, error: e });
        }
    }
    return handlerResult;
}
exports.pipRequirementsTxt = pipRequirementsTxt;
function projectTypeSupported(res) {
    return !('reason' in res);
}
async function isSupported(entity) {
    const remediationData = entity.testResult.remediation;
    if (!remediationData) {
        return { supported: false, reason: 'No remediation data available' };
    }
    // TODO: recursive inclusions?
    // TODO: fix the non null assertion here
    const fileName = entity.scanResult.identity.targetFile;
    const requirementsTxt = await entity.workspace.readFile(fileName);
    const { containsRequire } = await containsRequireDirective(requirementsTxt);
    if (containsRequire) {
        return {
            supported: false,
            reason: `Requirements with ${chalk.bold('-r')} or ${chalk.bold('-c')} directive are not yet supported`,
        };
    }
    if (!remediationData.pin || Object.keys(remediationData.pin).length === 0) {
        return {
            supported: false,
            reason: 'There is no actionable remediation to apply',
        };
    }
    return { supported: true };
}
exports.isSupported = isSupported;
/* Requires like -r, -c are not supported at the moment, as multiple files
 * would have to be identified and fixed together
 * https://pip.pypa.io/en/stable/reference/pip_install/#options
 */
async function containsRequireDirective(requirementsTxt) {
    const allMatches = [];
    const REQUIRE_PATTERN = new RegExp(/^[^\S\n]*-(r|c)\s+(.+)/, 'gm');
    const matches = getAllMatchedGroups(REQUIRE_PATTERN, requirementsTxt);
    for (const match of matches) {
        if (match && match.length > 1) {
            allMatches.push(match);
        }
    }
    return { containsRequire: allMatches.length > 0, matches: allMatches };
}
exports.containsRequireDirective = containsRequireDirective;
function getAllMatchedGroups(re, str) {
    const groups = [];
    let match;
    // tslint:disable-next-line:no-conditional-assignment
    while ((match = re.exec(str))) {
        groups.push(match);
    }
    return groups;
}
// TODO: optionally verify the deps install
async function fixIndividualRequirementsTxt(entity, options) {
    const fileName = entity.scanResult.identity.targetFile;
    const remediationData = entity.testResult.remediation;
    if (!remediationData) {
        throw new missing_remediation_data_1.MissingRemediationDataError();
    }
    if (!fileName) {
        throw new missing_file_name_1.MissingFileNameError();
    }
    const requirementsTxt = await entity.workspace.readFile(fileName);
    // TODO: allow handlers per fix type (later also strategies or combine with strategies)
    const { updatedManifest, changes } = update_dependencies_1.updateDependencies(requirementsTxt, remediationData.pin);
    if (!options.dryRun) {
        await entity.workspace.writeFile(fileName, updatedManifest);
    }
    return {
        original: entity,
        changes,
    };
}
exports.fixIndividualRequirementsTxt = fixIndividualRequirementsTxt;
//# sourceMappingURL=pip-requirements.js.map