"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProjectType = exports.isRequirementsTxtManifest = exports.pythonFix = void 0;
const debugLib = require("debug");
const pMap = require("p-map");
const micromatch = require("micromatch");
const ora = require("ora");
const load_handler_1 = require("./load-handler");
const supported_project_types_1 = require("./supported-project-types");
const debug = debugLib('snyk-fix:python');
async function pythonFix(entities, options) {
    const spinner = ora({ isSilent: options.quiet, stream: process.stdout });
    spinner.text = 'Looking for supported Python items';
    spinner.start();
    const pluginId = 'python';
    const handlerResult = {
        [pluginId]: {
            succeeded: [],
            failed: [],
            skipped: [],
        },
    };
    const entitiesPerType = {
        [supported_project_types_1.SUPPORTED_PROJECT_TYPES.REQUIREMENTS]: [],
    };
    for (const entity of entities) {
        const type = getProjectType(entity);
        if (!type) {
            const userMessage = `${entity.scanResult.identity.targetFile} is not supported`;
            debug(userMessage);
            handlerResult[pluginId].skipped.push({ original: entity, userMessage });
            continue;
        }
        entitiesPerType[type].push(entity);
    }
    spinner.succeed();
    await pMap(Object.keys(entitiesPerType), async (projectType) => {
        const projectsToFix = entitiesPerType[projectType];
        spinner.text = `Processing ${projectsToFix.length} ${projectType} items.`;
        spinner.render();
        try {
            const handler = load_handler_1.loadHandler(projectType);
            const { failed, skipped, succeeded } = await handler(projectsToFix, options);
            handlerResult[pluginId].failed.push(...failed);
            handlerResult[pluginId].skipped.push(...skipped);
            handlerResult[pluginId].succeeded.push(...succeeded);
        }
        catch (e) {
            debug(`Failed to fix ${projectsToFix.length} ${projectType} projects.\nError: ${e.message}`);
            handlerResult[pluginId].failed.push(...projectsToFix.map((p) => ({ original: p, error: e })));
        }
    }, {
        concurrency: 5,
    });
    spinner.succeed();
    return handlerResult;
}
exports.pythonFix = pythonFix;
function isRequirementsTxtManifest(targetFile) {
    return micromatch.isMatch(targetFile, 
    // micromatch needs **/* to match filenames that may include folders
    ['*req*.txt', 'requirements/*.txt', 'requirements*', '*.txt'].map((f) => '**/' + f));
}
exports.isRequirementsTxtManifest = isRequirementsTxtManifest;
function getProjectType(entity) {
    const targetFile = entity.scanResult.identity.targetFile;
    if (!targetFile) {
        return null;
    }
    const isRequirementsTxt = isRequirementsTxtManifest(targetFile);
    if (isRequirementsTxt) {
        return supported_project_types_1.SUPPORTED_PROJECT_TYPES.REQUIREMENTS;
    }
    return null;
}
exports.getProjectType = getProjectType;
//# sourceMappingURL=index.js.map